//! skadi's specific Error types.

use std::fmt::Display;

/// Each possible error raised by skadi functions.
pub enum SkadiError {
    /// Error while connecting to the Discord API.
    DiscordConnectionError(String),
    /// Error while connecting to the Matrix homeserver API.
    MatrixConnectionError(String),
}

/// The generic Result type for any skadi software component.
pub type Result<T> = std::result::Result<T, SkadiError>;

impl Display for SkadiError {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        let message: &str = match self {
            SkadiError::DiscordConnectionError(error) => error,
            SkadiError::MatrixConnectionError(error) => error,
        };

        write!(f, "{}", message)
    }
}
