//! skadi is a bridge between Discord and Matrix.

use connection::{connect_discord, connect_matrix};
use log::{error, info};
use std::{env, process::ExitCode};
use tokio::sync::oneshot;

mod connection;
mod error;

/// skadi's crate name.
const CRATE_NAME: &str = env!("CARGO_PKG_NAME");
/// skadi's crate version.
const CRATE_VERSION: &str = env!("CARGO_PKG_VERSION");

#[tokio::main]
async fn main() -> Result<(), ExitCode> {
    env_logger::init();

    let (tx_discord, rx_discord) = oneshot::channel();
    let (tx_matrix, rx_matrix) = oneshot::channel();

    info!("Starting {CRATE_NAME} v{CRATE_VERSION}");

    tokio::spawn(async {
        if let Err(error) = connect_discord().await {
            error!("Failed to connect to Discord: {error}");
        };
        let _ = tx_discord.send(());
    });

    tokio::spawn(async {
        if let Err(error) = connect_matrix().await {
            error!("Failed to connect to Matrix: {error}");
        }
        let _ = tx_matrix.send(());
    });

    tokio::select! {
        _ = rx_discord => error!("Discord connection dropped."),
        _ = rx_matrix => error!("Matrix connection dropped."),
    };

    Err(ExitCode::FAILURE)
}
